@@ -2,7 +2,6 @@ import abc
 import threading
 
 from .abstract import TopKQueryResult
-from .exceptions import FutureTimeoutError
 from .types import Status
 
 
@@ -43,12 +42,14 @@ class Future(AbstractFuture):
         self._canceled = False
         self._done = False
         self._response = None
+        self._results = None
         self._exception = None
 
         self.__init()
 
     def __del__(self):
-        pass
+        self._future.__del__()
+        self._future = None
         # self._future.__del__()
 
     @abc.abstractmethod
@@ -65,19 +66,19 @@ class Future(AbstractFuture):
 
         def async_done_callback(future):
             with self._condition:
-                self._response = future.result()
                 # delete gRCP future manually
-                self._future.__del__()
-                self._future = None
+                # self._future.__del__()
+                # self._future = None
 
                 # If user specify done callback function, execute it.
                 try:
+                    self._response = future.result()
+                    self._results = self.on_response(self._response)
                     if self._done_cb:
-                        results = self.on_response(self._response)
-                        if isinstance(results, tuple):
-                            self._done_cb(*self.on_response(self._response))
+                        if isinstance(self._results, tuple):
+                            self._done_cb(*self._results)
                         else:
-                            self._done_cb(self.on_response(self._response))
+                            self._done_cb(self._results)
                 except Exception as e:
                     self._exception = e
                 finally:
@@ -91,7 +92,8 @@ class Future(AbstractFuture):
         with self._condition:
             # future not finished. wait callback being called.
             to = kwargs.get("timeout", None)
-            self._response = self._future.result(timeout=to)
+            if not self._future.done() or not self._response:
+                self._response = self._future.result(timeout=to)
             # if not self._done and not self._canceled:
             #     to = kwargs.get("timeout", None)
             #     self._condition.wait(to)
@@ -106,7 +108,10 @@ class Future(AbstractFuture):
             # just return response object received from gRPC
             return self._response
 
-        return self.on_response(self._response)
+        if self._results:
+            return self._results
+        else:
+            return self.on_response(self._response)
 
     def cancel(self):
         with self._condition:
@@ -116,12 +121,17 @@ class Future(AbstractFuture):
             #     self._canceled = True
             self._condition.notify_all()
 
-    def done(self):
-        self.exception()
-        with self._condition:
+    def is_done(self):
+        return self._done
 
+    def done(self):
+        # self.exception()
+        with self._condition:
             if self._future and not self._future.done():
-                self._future.result()
+                try:
+                    self._future.result()
+                except Exception as e:
+                    self._exception = e
 
             self._condition.notify_all()
 
